// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const Lang = imports.lang;
const St = imports.gi.St;

const Main = imports.ui.main;
const Tweener = imports.ui.tweener;

const PressureViz = new Lang.Class({
    Name: 'PressureViz',

    _init: function(threshold) {
        this.actor = new St.Widget({ style_class: 'pressure-viz',
                                     x_expand: true,
                                     y_expand: true,
                                     opacity: 0 });
        this._threshold = threshold;
        this.value = 0;
    },

    get value() {
        return this._value;
    },

    set value(value) {
        value = Math.min(value, this._threshold);

        this._value = value;
        let percent = value / this._threshold;

        this.actor.opacity = percent * 255;
        let y = percent * 75;
        this.actor.height = y;
        this.actor.translation_y = -y;
    },

    destroy: function() {
        this.actor.destroy();
    },
});

let trayBarrierHitId = 0;
let trayPressureViz = null;
let origReset = null;

function enable() {
    let pressure = Main.layoutManager._trayPressure;

    trayPressureViz = new PressureViz(pressure._threshold);
    Main.layoutManager.trayBox.add_child(trayPressureViz.actor);

    trayBarrierHitId = Main.layoutManager._trayBarrier.connect('hit', function() {
        if (pressure._isTriggered)
            return;

        Tweener.removeTweens(trayPressureViz);
        trayPressureViz.value = pressure._currentPressure;
        Tweener.addTween(trayPressureViz, { time: pressure._timeout / 1000,
                                            onUpdate: function() {
                                                pressure._lastTime = global.display.get_current_time_roundtrip();
                                                trayPressureViz.value = pressure._currentPressure;
                                                pressure._trimBarrierEvents();
                                            } });
    });

    origReset = pressure._reset;
    pressure._reset = function() {
        origReset();
        Tweener.removeTweens(trayPressureViz);
        Tweener.addTween(trayPressureViz, { value: 0, time: 0.5 });
    };
}

function disable() {
    let pressure = Main.layoutManager._trayPressure;
    trayPressureViz.destroy();
    trayPressureViz = null;
    pressure._reset = origReset;
    origReset = null;
    Main.layoutManager._trayBarrier.disconnect(trayBarrierHitId);
    trayBarrierHitId = 0;
}

