
const GLib = imports.gi.GLib;
const GWeather = imports.gi.GWeather;
const GnomeDesktop = imports.gi.GnomeDesktop;
const Gtk = imports.gi.Gtk;
const Gio = imports.gi.Gio;
const Atk = imports.gi.Atk;

let Me, now_utc;

if (ARGV[0] == '--localize') {
  if (ARGV[1]) {
    GLib.setenv('LC_ALL', ARGV[1], true);
    GLib.unsetenv('LANGUAGE');
  }
  Gtk.init(null);

  print(JSON.stringify(getRCodes()));
}
else {
  Gtk.init(null);

  print(JSON.stringify(getLocalCache()));
}

function get_current_file() {
    return (new Error()).stack.split('\n')[0].match(/@(.+):\d+/)[1];
}

function showProgress(ui, text, c, n, sb, flip) {
  let pbar = ui.get_object('progress-bar');
  if (!text) {
    pbar.pulse();
  }
  else {
    let statusbar = ui.get_object('progress-sbar');
    if (c < 0) { c = n + c; }
    pbar.text = text + (n ? ' (' + (c+1) + '/' + n + ')' : '');
    pbar.fraction = (n ? c/n : c);
    if (sb) {
      statusbar.pop(0); statusbar.push(0, "@ " + sb);
    }
    if (flip) {
      pbar.inverted = !pbar.inverted;
    }
  }
  while (Gtk.events_pending()) Gtk.main_iteration();
}

function fillLocalValues(ui) {
  let localValues = {};
  let succ = false, val;
  let cache_file = Me.dir.get_child('_locations.json');
  try {
    [succ, val] = GLib.file_get_contents(cache_file.get_path());
  }
  catch (e) {}
  if (succ) {
    let obj = JSON.parse(val);
    if (obj.ver == 1) { localValues = obj.locales; }
    else { succ = false; }
  }

  if (!succ) {
    localValues = getGlobalCache(ui);
    try {
      GLib.file_set_contents(cache_file.get_path(),
			     JSON.stringify({ ver: 1, locales: localValues }));
    }
    catch (e) { log("error: " + e); }
  }
  return localValues;
}

function set_timezones(i, z) {
  let ltz = i.get_timezone();
  let tz_names = [];
  let tz_offsets = [];
  let tzid = [];
  if (ltz) {
    tzid = i.get_timezone().get_tzid();
    let tz = GLib.TimeZone.new(tzid);
    let ivs = [ tz.find_interval(GLib.TimeType.UNIVERSAL, now_utc.to_unix()) ];
    for (let w = 1; w <= 53; w += 1) {
      let iv = tz.find_interval(GLib.TimeType.UNIVERSAL, now_utc.add_weeks(w).to_unix());
      if (!ivs.some(function (e) e == iv)) { ivs.push(iv); }
    }
    ivs.forEach(function (e) {
      let abbrev = tz.get_abbreviation(e);
      if (!tz_names.some(function (f) f == abbrev)) { tz_names.push(abbrev); }
      let off_sec = tz.get_offset(e);
      let pfx = '+';
      if (off_sec < 0) { pfx = '-'; off_sec = -off_sec; }
      let off_min = ~~(off_sec / 60);
      let off = ~~(off_min / 60);
      let off_r = off_min % 60;
      let offset_name = 'UTC' + pfx + off + ':' + (off_r ? off_r : '00') + (tz.is_dst(e) ? '*' : '');
      if (!tz_offsets.some(function (f) f == offset_name)) { tz_offsets.push(offset_name); }
    });
  }
  z.tz_names = tz_names;
  z.tz_offsets = tz_offsets;
  z.i2 = [tzid];
}

function add_global_infos(i, z) {
  z.cc = i.get_country();

  set_timezones(i, z);
}

function get_all_stations(world) {
  let ncode = [world];
  let codes = {};
  while (ncode.length) {
    ncode = ncode.map(function (x) {
      let c = x.get_code();
      if (c) {
	codes[c] = x;
	return [];
      }
      return x.get_children();
    }).reduce(function (a, b) {
      return a.concat(b);
    });
  }
  return codes;
}

function getRCodes() {
  let world = GWeather.Location.new_world(true);
  let codes = get_all_stations(world);
  let rcodes = {}, x;
  now_utc = GLib.DateTime.new_now_utc();
  for (x in codes) {
    let i = codes[x].get_parent();
    let i_codes = i.get_children()
	  .sort(function (a, b) a.get_distance(a.get_parent()) - b.get_distance(b.get_parent()))
	  .map(function (e) e.get_code());
    if (i_codes[0] == x) {
      let z = { d: [] };
      if (ARGV[1] == 'C.utf8') {
	add_global_infos(i, z);
      }
      while (i.get_level() >= GWeather.LocationLevel.COUNTRY) {
	z.d.push(i.get_name());
	i = i.get_parent();
      }
      rcodes[x] = z;
    }
  }
  return rcodes;
}

function getGlobalCache(ui) {
  let localValues = {};
  let locales = GnomeDesktop.get_all_locales()
	.filter(function (e) { return e.match(/[.]utf8/); });
  locales.push('C.utf8');
  let i = locales.length;
  locales.forEach(function (e) {
    let [succ, pid, pin, pout, perr] = GLib.spawn_async_with_pipes(
      null, ['gjs-console', Me.file, '--localize', e],
      null, GLib.SpawnFlags.SEARCH_PATH, null);
    showProgress(ui, 'Querying locale', -i, locales.length, e);
    let channel = GLib.IOChannel.unix_new(pout);
    channel.set_close_on_unref(true);
    localValues[e] = JSON.parse(channel.read_to_end()[1]);
    i -= 1;
  });
  return localValues;
}

function fillCodeMap(ui) {
  let locale = Atk.NoOpObject.new(Gtk.Invisible.new()).get_object_locale();
  let succ = false, val;
  let cache_file = Me.dir.get_child('_locations_cache_' + locale + '.json');
  try {
    [succ, val] = GLib.file_get_contents(cache_file.get_path());
  }
  catch (e) {}
  if (succ) {
    let obj = JSON.parse(val);
    if (obj.ver == 1) { return obj.codemap; }
  }

  let localValues = fillLocalValues(ui);

  let localCodes = localValues[locale] || localValues['C.utf8'];
  let cLangCodes = localValues['C.utf8'];
    
  let code_, lang_;
  let num_codes = Object.getOwnPropertyNames(localValues).length;
  let i = num_codes;

  showProgress(ui, 'Parsing locales', 0, null, ' ', true);
  for (code_ in localCodes) {
    let cr = localCodes[code_];
    let ccr = cLangCodes[code_];
    cr.i2 = cr.d.concat(ccr.tz_names).concat(ccr.tz_offsets);
    cr.cc = ccr.cc;
    cr.d2 = [];
    cr.tz_names = ccr.tz_names;
    cr.tz_offsets = ccr.tz_offsets;
  }
  for (lang_ in localValues) {
    showProgress(ui, 'Parsing locale', -i, num_codes, lang_);
    i -= 1;
    if (lang_ != locale) {
      let langCodes = localValues[lang_];
      for (code_ in langCodes) {
	let val = langCodes[code_].d;
	let cVal = cLangCodes[code_].d;
	let map = localCodes[code_];
	let [,cc] = lang_.match(/_(.+)[.]/) || [];
	if (map && cc && map.cc == cc) {
	  val.forEach(function (e) {
	    if (!map.i2.some(function (f) f == e)) {
	      map.i2.push(e);
	    }
	  });
	  let rv = val[0];
	  if (rv != map.d[0] && rv != cVal[0] &&
	      !map.d2.some(function (f) f == rv)) {
	    map.d2.push(rv);
	  }
	}
      }
    }
  }
  try {
    succ = GLib.file_set_contents(cache_file.get_path(),
				  JSON.stringify({ ver: 1, codemap: localCodes }));
  }
  catch (e) { log("error: " + e); }
  return localCodes;
}

function getLocalCache() {
  Me = get_script_infos();
  let ui = new Gtk.Builder();
  ui.add_from_file(Me.dir.get_child('cache.ui').get_path());
  ui.get_object('progress').show();

  let cachemap = fillCodeMap(ui);
  
  ui.get_object('progress').hide();
  return cachemap;
}

function get_script_infos() {
  let file = get_current_file();
  let gio_file = Gio.File.new_for_path(file);
  let dir = gio_file.get_parent();
  return {
    file: file,
    dir: dir
  };
}
